"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const FMath = require("./fmath");
/**
 * Coordinate dimensions used in the vector types.
 */
exports.x = 0;
exports.y = 1;
exports.z = 2;
exports.w = 3;
/**
 * Initialize a zero vector with dim components.
 * @param dim Vector dimension.
 */
function zero(dim) {
    return unif(dim, 0);
}
exports.zero = zero;
/**
 * Initializes a vector with the same value in all components.
 * @param dim Vector dimension.
 * @param value Value for components.
 */
function unif(dim, value) {
    return new Array(dim).fill(value);
}
exports.unif = unif;
/**
 * Return a copy of the input vector but with different dimension.
 * @param vec The input vector.
 * @param dim The dimension of the new vector.
 * @param pad The value used to pad the new vector, if dimension is greater
 * than originally.
 */
function redim(vec, dim, pad = 1) {
    let len = vec.length;
    let res = new Array(dim);
    for (let i = 0; i < dim; ++i)
        res[i] = i < len ? vec[i] : pad;
    return res;
}
exports.redim = redim;
/**
 * Returns the vector dimension, i.e. number of components.
 */
function dimension(v) {
    return v.length;
}
exports.dimension = dimension;
/**
 * Return one or more components of the vector in arbitrary order. The components
 * returned depend on the dimensions specified in the coords argument. Note that
 * the same component can occur multiple times in coords. So, it is valid to call
 * the function like this:
 *
 * swizzle ([x, x, y])
 */
function swizzle(v, ...coords) {
    let len = coords.length;
    let res = new Array(len);
    for (let i = 0; i < len; ++i)
        res[i] = v[coords[i]];
    return res;
}
exports.swizzle = swizzle;
/**
 * The lenght of the vector squared. Faster to calculate than the actual length,
 * and useful for comparing vector magnitudes.
 */
function lenSqr(v) {
    let res = 0;
    let len = v.length;
    for (let i = 0; i < len; ++i)
        res += v[i] * v[i];
    return res;
}
exports.lenSqr = lenSqr;
/**
 * Length of the vector.
 */
function len(v) {
    return Math.sqrt(lenSqr(v));
}
exports.len = len;
/**
 * Invert a vector.
 * @param v Vector to be inverted.
 * @param out Result vector.
 */
function inv(v, out = new Array(v.length)) {
    let len = v.length;
    for (let i = 0; i < len; ++i)
        out[i] = -v[i];
    return out;
}
exports.inv = inv;
/**
 * Add the second vector to the first one componentwise.
 * @param v Input vector.
 * @param other Vector or number to be added.
 * @param out Result vector.
 */
function add(v, other, out = new Array(v.length)) {
    let len = v.length;
    if (typeof other === 'number')
        for (let i = 0; i < len; ++i)
            out[i] = v[i] + other;
    else
        for (let i = 0; i < len; ++i)
            out[i] = v[i] + other[i];
    return out;
}
exports.add = add;
/**
 * Subtract the second vector from the first one componentwise.
 * @param v Input vector.
 * @param other Vector or number to be subtracted.
 * @param out Result vector.
 */
function sub(v, other, out = new Array(v.length)) {
    let len = v.length;
    if (typeof other === 'number')
        for (let i = 0; i < len; ++i)
            out[i] = v[i] - other;
    else
        for (let i = 0; i < len; ++i)
            out[i] = v[i] - other[i];
    return out;
}
exports.sub = sub;
/**
 * Multiply the first vector with the second one componentwise.
 * @param v Input vector.
 * @param other Multiplier vector or number.
 * @param out Result vector.
 */
function mul(v, other, out = new Array(v.length)) {
    let len = v.length;
    if (typeof other === 'number')
        for (let i = 0; i < len; ++i)
            out[i] = v[i] * other;
    else
        for (let i = 0; i < len; ++i)
            out[i] = v[i] * other[i];
    return out;
}
exports.mul = mul;
/**
 * Divide the first vector by the second one componentwise.
 * @param v Input vector.
 * @param other Divider vector or number.
 * @param out Result vector.
 */
function div(v, other, out = new Array(v.length)) {
    let len = v.length;
    if (typeof other === 'number')
        for (let i = 0; i < len; ++i)
            out[i] = v[i] / other;
    else
        for (let i = 0; i < len; ++i)
            out[i] = v[i] / other[i];
    return out;
}
exports.div = div;
/**
 * Normalize a vector.
 */
function norm(v, out = new Array(v.length)) {
    let l = len(v);
    if (l == 0)
        throw RangeError("Cannot normalize zero vector");
    let vlen = v.length;
    for (let i = 0; i < vlen; ++i)
        out[i] = v[i] / l;
    return out;
}
exports.norm = norm;
/**
 * Checks if the first vector is same as the second one.
 * @param v1 First vector
 * @param v2 Second vector
 */
function equals(v1, v2) {
    let len = v1.length;
    if (len != v2.length)
        return false;
    for (let i = 0; i < len; ++i)
        if (v1[i] != v2[i])
            return false;
    return true;
}
exports.equals = equals;
/**
 * Checks if the first vector is approximately same as the second one.
 * @param v1 First vector
 * @param v2 Second vector
 * @param epsilon The maximum allowed difference of any component.
 */
function approxEquals(v1, v2, epsilon) {
    let len = v1.length;
    if (len != v2.length)
        return false;
    for (let i = 0; i < len; ++i)
        if (!FMath.approxEquals(v1[i], v2[i], epsilon))
            return false;
    return true;
}
exports.approxEquals = approxEquals;
/**
 * Return the dot product of two vectors.
 * @param v Input vector.
 * @param other Other vector of the dot product.
 */
function dot(v, other) {
    let res = 0;
    let len = v.length;
    for (let i = 0; i < len; ++i)
        res += v[i] * other[i];
    return res;
}
exports.dot = dot;
/**
 * Return the cross product of two vectors. The vectors must be 3-dimensional.
 * @param v Input vector.
 * @param other The other vector of the cross product.
 * @param out Result vector.
 */
function cross(v, other, out = new Array(v.length)) {
    if (v.length != 3 || other.length != 3)
        throw RangeError(`Both vectors must 3-dimensional.`);
    out[exports.x] = v[exports.y] * other[exports.z] - v[exports.z] * other[exports.y];
    out[exports.y] = v[exports.z] * other[exports.x] - v[exports.x] * other[exports.z];
    out[exports.z] = v[exports.x] * other[exports.y] - v[exports.y] * other[exports.x];
    return out;
}
exports.cross = cross;
/**
 * Replace components of the vector with their absolute values.
 * @param v Input vector.
 * @param out Result vector.
 */
function abs(v, out = new Array(v.length)) {
    let len = v.length;
    for (let i = 0; i < len; ++i)
        out[i] = Math.abs(v[i]);
    return out;
}
exports.abs = abs;
/**
 * Run the components of the vector through Math.floor function.
 * @param v Input vector.
 * @param out Result vector.
 */
function floor(v, out = new Array(v.length)) {
    let len = v.length;
    for (let i = 0; i < len; ++i)
        out[i] = Math.floor(v[i]);
    return out;
}
exports.floor = floor;
/**
 * Run the components of the vector through Math.ceil function.
 * @param v Input vector.
 * @param out Result vector.
 */
function ceil(v, out = new Array(v.length)) {
    let len = v.length;
    for (let i = 0; i < len; ++i)
        out[i] = Math.ceil(v[i]);
    return out;
}
exports.ceil = ceil;
/**
 * Run the components of the vector through Math.round function.
 * @param v Input vector.
 * @param out Result vector.
 */
function round(v, out = new Array(v.length)) {
    let len = v.length;
    for (let i = 0; i < len; ++i)
        out[i] = Math.round(v[i]);
    return out;
}
exports.round = round;
/**
 * Run the components of the vector through FMath.fract function.
 * @param v Input vector.
 * @param out Result vector.
 */
function fract(v, out = new Array(v.length)) {
    let len = v.length;
    for (let i = 0; i < len; ++i)
        out[i] = FMath.fract(v[i]);
    return out;
}
exports.fract = fract;
/**
 * Calculate the minimum of two vectors componentwise.
 * @param v The input vector.
 * @param other The vector to compare with.
 * @param out Result vector.
 */
function min(v, other, out = new Array(v.length)) {
    let len = v.length;
    for (let i = 0; i < len; ++i)
        out[i] = Math.min(v[i], other[i]);
    return out;
}
exports.min = min;
/**
 * Calculate the maximum of two vectors componentwise.
 * @param v The input vector.
 * @param other The vector to compare with.
 * @param out Result vector.
 */
function max(v, other, out = new Array(v.length)) {
    let len = v.length;
    for (let i = 0; i < len; ++i)
        out[i] = Math.max(v[i], other[i]);
    return out;
}
exports.max = max;
/**
 * Clamp the components of a vector to a given range.
 * @param v The input vector.
 * @param min Minimum component value.
 * @param max Maximum component value.
 * @param out Result vector.
 */
function clamp(v, min, max, out = new Array(v.length)) {
    let len = v.length;
    for (let i = 0; i < len; ++i)
        out[i] = FMath.clamp(v[i], min, max);
    return out;
}
exports.clamp = clamp;
/**
 * Calculate the interpolated vector in a given position [0, 1].
 * @param v The input vector.
 * @param other The vector to be interpolated to.
 * @param interPos The position between 0 and 1, zero representing v and one other.
 * @param out Result vector.
 */
function mix(v, other, interPos, out = new Array(v.length)) {
    let len = v.length;
    for (let i = 0; i < len; ++i)
        out[i] = FMath.mix(v[i], other[i], interPos);
    return out;
}
exports.mix = mix;
/**
 * Return a vector of zeros and ones depending if the input vector components are
 * greater or less than the edge value.
 * @param v The input vector.
 * @param edge The edge to which the components are compared.
 * @param out Result vector.
 */
function step(v, edge, out = new Array(v.length)) {
    let len = v.length;
    for (let i = 0; i < len; ++i)
        out[i] = FMath.step(v[i], edge);
    return out;
}
exports.step = step;
/**
 * Return a vector with components in range [0, 1] depending on how close the
 * input vector values are to the lower and upper edge value. If a component is
 * less that lower edge it gets value 0. Conversely, if it is greater than the
 * upper edge, it gets value 1. If a component is between lower and upper edge,
 * its value is smoothly interpolated between zero and one.
 * @param v The input vector.
 * @param edgeLower The lower edge to which the components are compared.
 * @param edgeUpper The upper edge to which the components are compared.
 * @param out Result vector.
 */
function smoothStep(v, edgeLower, edgeUpper, out = new Array(v.length)) {
    let len = v.length;
    for (let i = 0; i < len; ++i)
        out[i] = FMath.smoothStep(v[i], edgeLower, edgeUpper);
    return out;
}
exports.smoothStep = smoothStep;
/**
 * Returns the string representation of a vector formatted like this: [x y z]
 */
function toString(v) {
    return "[" + v.join(" ") + "]";
}
exports.toString = toString;
